An introduction to dimension reduction in nonparametric kernel regression
-------------------------------------------------------------------------
  
  
n <- 100
x <- runif(n,0,1)
epsilon <- rnorm(n,0,sd=1/9)
y <- sin(4*x)+epsilon
# Cross-validation function
CV.prog <- function(x,y,hmin=(max(x)-min(x))/20,
                    hmax=(max(x)-min(x))/2,nbh=25,graph=TRUE){
  n <- length(x)
  vecth <- seq(from=hmin,to=hmax,length=nbh)
  matCV <- cbind(vecth,rep(0,nbh))
  for (h in 1:nbh){
    ypred <- rep(0,n)
    for (i in 1:n){
      ypred[i] <- ksmooth(x[-i],y[-i],kernel="normal",
                          band=vecth[h],x.points=x[i])$y
    }
    matCV[h,2] <- sum((y-as.matrix(ypred,ncol=1))^2)
  }
  hopt <- matCV[which(matCV[,2]==min(matCV[,2],na.rm=TRUE)),1]
  if (graph==TRUE) {
    plot(matCV,xlab="h",ylab="CV(h)")
    abline(v=hopt,col=2,lwd=3)
  }
  list(matCV=matCV,hopt=hopt)
}
# Estimation of the link function via kernel smoothing :
# we use "ksmooth" function with a Gaussian kernel
# and the optimal CV bandwidth.
par(mfrow=c(1,2)) # split the graphical device in one row
# and two columns
resCV <- CV.prog(x,y)
resKhopt <- ksmooth(x,y,kernel="normal",bandwidth=resCV$hopt)
plot(x,y,pch=4) # scatterplot of the data
# plot of the true link function (blue line)
lines(sort(x),sin(4*sort(x)),col=4,lwd=3)
# plot of the estimated link function (green line)
lines(resKhopt,col=3,lwd=3)


## Application on simulated data
# Simulation of the data
n <- 200
p <- 5
beta <- matrix(c(1,2,-1,-2,0),ncol=1)
X <- matrix(runif(p*n),ncol=p)
index <- X%*%beta # true index
epsilon <- rnorm(n,0,0.5)
y <- (index)^3+epsilon
matYX <- cbind(y,X) # dataset
dimnames(matYX) <- list(c(1:n),c("y","x1","x2","x3","x4","x5"))
pairs(matYX,pch=4) # matrix of scatterplots
# Plot of true index versus Y
plot(index,y,xlab="True index",ylab="y",pch=4)
u<-seq(min(index),max(index),length=50)
lines(u,u^3,,col=4,lwd=3)
library(edrGraphicalTools) # Loads the package
# Compute the EDR direction using SIR with H=5 slices
# Note that the option K=1 is only for visualization purpose.
resSIR <- edr(Y=y,X=X,H=5,K=1,method="SIR-I")
# Eigenvalue screeplot
plot(resSIR$eigvalEDR,xlab=" ",ylab="Eigenvalue")
# Display the first estimated EDR direction
resSIR
# Plots of first estimated index versus Y and
# second estimated index versus Y
indexpred1 <- X%*%resSIR$matEDR[,1]
indexpred2 <- X%*%resSIR$matEDR[,2]
par(mfrow=c(1,2))
plot(indexpred1,y,xlab="First estimated index",ylab="y",pch=4)
plot(indexpred2,y,xlab="Second estimated index",ylab="y",pch=4)
# Plot of the true index versus the first estimated index
par(mfrow=c(1,1))
plot(index,indexpred1)
title(paste("correlation=",round(cor(index,indexpred1),digit=3)))
# Select the bandwidth with the cross-validation function
# using the first estimated index and Y
resCVsir <- CV.prog(indexpred1,y)
# Estimation and plot of the link function
# using the first estimated index and Y
res1Khopt <- ksmooth(indexpred1,y,kernel="normal",
                     bandwidth=resCVsir$hopt)
plot(indexpred1,y,xlab="First estimated index",ylab="y",pch=4)
lines(res1Khopt,col=3,lwd=3)

## Application on astronomical data


data=read.table('vagc.txt',header=TRUE)
# delete missing data
data=na.omit(data)
y=as.numeric(data[,43])
x=as.matrix(data[,c(2:42,45:50)])
centre<-apply(x,2,mean)
x.centre<-sweep(data.matrix(x),2,centre)
dat=data.frame(y,x)

datanew=read.table('vagc2.txt',header=TRUE)
ynew=as.numeric(datanew[,43])
xnew=as.matrix(datanew[,c(2:42,45:50)])
centrenew<-apply(xnew,2,mean)
xnew.centre<-sweep(data.matrix(xnew),2,centrenew)
datnew=data.frame(y,x)

x<-x.centre
xnew<-xnew.centre

library(edrGraphicalTools)
# the matrices x and xnew are centered.
# Step 1a: computation of SIR-QZ on the whole set of covariates
resSIR <- edr(Y=y,X=x,H=10,K=1,method="SIR-I",submethod="SIR-QZ")
index1 <- x%*%resSIR$matEDR[,1]
index2 <- x%*%resSIR$matEDR[,2]
plot(index1,y,pch=4,xlab="First estimated index",ylab="Y")
plot(index2,y,pch=4,xlab="Second estimated index",ylab="Y")
# Step 1b: estimation of the link function via kernel smoothing
resCV <- CV.prog(index1,y)
resKhopt <- ksmooth(index1,y,kernel="normal",bandwidth=resCV$hopt)
# Step 1c: computation of the RMSE on the test sample
index1new <- xnew%*%resSIR$matEDR[,1]
Resnewpred <- ksmooth(index1,y,kernel="normal",bandwidth=resCV$hopt,x.points=index1new)
ynewpred <- Resnewpred$y
# Warning: this vector is "sorted" accordingly to the x.points vector
# ynew.sorted has thus to be sorted accordingly.
ynew.sorted <- ynew[sort(index1new,index.return=TRUE)$ix]
RMSE1 <- sqrt(mean((ynew.sorted-ynewpred)^2))
# Step 2a: selection of a subset of covariates via CSS procedure
res2<-edrSelec(Y=y,X=x,H=10,K=1,"CSS",pZero=10,NZero=50000,zeta=0.05)
plot(res2) # plot of the CSS output
x.simple<-x[,c(41,3,2)] # matrix of selected covariates
# Step 2b: computation of SIR on the subset of selected covariates
resSIRsimple <- edr(Y=y,X=x.simple,H=10,K=1,method="SIR-I")
index1simple<-x.simple%*%resSIRsimple$matEDR[,1]
# plot of the estimated indices obtained in steps 1a and 2b
plot(index1, index1simple,pch=4,xlab="First estimated index (first step)",ylab="First estimated index (second step)")
cor(index1, index1simple) # linear correlation between these indices
# Step 2c: kernel estimation of the link function
resCVsimple <- CV.prog(index1simple,y)
resKhoptsimple <- ksmooth(index1simple,y,kernel="normal",bandwidth=resCVsimple$hopt)
# Step 2d: computation of the RMSE on the test sample
xnew.simple<-xnew[,c(41,3,2)]
index1newsimple<-xnew.simple%*%resSIRsimple$matEDR[,1]
Resnewpredsimple<-ksmooth(index1simple,y,kernel="normal",bandwidth=resCVsimple$hopt,x.points=index1newsimple)
ynewpredsimple<-Resnewpredsimple$y
ynew.sortedsimple<-ynew[sort(index1newsimple,index.return=TRUE)$ix]
RMSE2 <- sqrt(mean((ynew.sortedsimple-ynewpredsimple)^2))
# Step 2e: plot of the link function on the training and test samples
range.y <- c(min(y,ynew),max(y,ynew))
range.x <- c(min(index1simple, index1newsimple),max(index1simple, index1newsimple))
plot(index1simple,y,pch=4,xlim=range.x,ylim=range.y,col=1,xlab="Estimated index",ylab="Y")
lines(resKhoptsimple,col=3,lwd=3)
par(new=TRUE)
plot(index1newsimple,ynew,pch=4,xlim=range.x,ylim=range.y,col=2,xlab="Estimated index",ylab="Y")
par(new=FALSE)

