Spatial patterns analysis in cosmology based on marked point processes
----------------------------------------------------------------------
  
  
#
# Clean memory
#
rm(list=ls())

#
# Set working directory
#
setwd("~/DOCS/TALKS/ANNECY/R")


# 
# Load libraries
#
library(spatstat)

#
# Read and print data : to be included in the presentation but not only ...
#
data(cells)
plot(cells)
x11()
plot(allstats(cells),main="Summary statistics - cells")

x11()
par(mfrow=c(2,2),pty="s")
plot(Fest(cells),main="F function - cells")
plot(Gest(cells),main="G function - cells")
plot(Jest(cells),main="J function - cells")
plot(Kest(cells),main="K function - cells")


data(redwoodfull)
plot(redwoodfull)
x11()
par(mfrow=c(2,2),pty="s")
plot(Fest(redwoodfull),main="F function - redwoodfull")
plot(Gest(redwoodfull),main="G function - redwoodfull")
plot(Jest(redwoodfull),main="J function - redwoodfull")
plot(Kest(redwoodfull),main="K function - redwoodfull")


data(japanesepines)
plot(japanesepines)

x11()
par(mfrow=c(2,2),pty="s")
plot(Fest(japanesepines),main="F function - japanesepines")
plot(Gest(japanesepines),main="G function - japanesepines")
plot(Jest(japanesepines),main="J function - japanesepines")
plot(Kest(japanesepines),main="K function - japanesepines")

x11()
par(mfrow=c(1,3),pty="s")
plot(Jest(cells),main="J function - cells")
plot(Jest(redwoodfull),main="J function - redwoodfull")
plot(Jest(japanesepines),main="J function - japanesepines")


data(amacrine)
plot(amacrine,cols=c("blue","red"))

data(copper)
attach(copper)
L=rotate(Lines,pi/2)
P=rotate(Points,pi/2)
plot(L,main="Copper",col="blue")
points(P$x,P$y,col="red")

#
# Simulate a Strauss process
#
mo = list(cif = "strauss", par = list(beta = 2, gamma = 0.2, r = 0.7),w = square(10))
X = rmh(model = mo, start = list(n.start = 42), control = list(nrep = 1e+06))
plot(X)

#
# Simulate a multi-type Strauss and print its sufficient statistics
#

computeSufficientStatistics = function(X,r)
#
# This functions computes the sufficient statistics of a three types point pattern X.
# The sufficient statistics are the number of points of each type and the number of pairs
# of points of different type, that are closer than r.
#
{
# Select the different patterns and unmark them
i1=which(X$marks=="A")
i2=which(X$marks=="B")
i3=which(X$marks=="C")

d.mat=pairdist(X,method="C")

d1=d.mat[i1,c(i2,i3)]
dim.d1=dim(d1)
if(dim.d1[1]<dim.d1[2]) { d1=t(d1) }
d1.cond=(d1<=r)
d1.sel=lower.tri(d1,diag=TRUE)
np23=sum(d1.cond & d1.sel)

d2=d.mat[i2,c(i3,i1)]
dim.d2=dim(d2)
if(dim.d2[1]<dim.d2[2]){ d2=t(d2) }
d2.cond=(d2<=r)
d2.sel=lower.tri(d2,diag=TRUE)
np31=sum(d2.cond & d2.sel)

d3=d.mat[i3,c(i1,i2)]
dim.d3=dim(d3)
if(dim.d3[1]<dim.d3[2]){ d3=t(d3) }
d3.cond=(d3<=r)
d3.sel=lower.tri(d3,diag=TRUE)
np12=sum(d3.cond & d3.sel)

list(n1=length(i1),n23=np23,n2=length(i2),n31=np31,n3=length(i3),n12=np12)
}

rp=matrix(c(0.05,0.05,0.05,0.05,0.05,0.05,0.05,0.05,0.05),nrow=3,ncol=3)
bp=300
gp=matrix(c(1,0.5,0.5,0.5,1,0.5,0.5,0.5,1),nrow=3,ncol=3)
mStrauss=rmhmodel(cif="straussm",par=list(beta=c(bp,bp,bp),gamma=gp,radii=rp),w=square(1),types=c("A","B","C"))

n.iter=500
n.moves=1000
radius=0.05
sufficient.stat=matrix(0,nrow=6,ncol=n.iter)

xinit=rmh(mStrauss,start=list(n.start=0),control=list(nrep=10*n.moves))
for (i in 1 : n.iter)
{
 xfin=rmh(mStrauss,start=list(x.start=xinit),control=list(nrep=n.moves))
 sstat=computeSufficientStatistics(xfin,radius)
 sufficient.stat[1,i]=sstat$n1
 sufficient.stat[2,i]=sstat$n23
 sufficient.stat[3,i]=sstat$n2
 sufficient.stat[4,i]=sstat$n31
 sufficient.stat[5,i]=sstat$n3
 sufficient.stat[6,i]=sstat$n12
 xinit=xfin
}

n1.average=cumsum(sufficient.stat[1,])/(1:n.iter)
n23.average=cumsum(sufficient.stat[2,])/(1:n.iter)
n2.average=cumsum(sufficient.stat[3,])/(1:n.iter)
n31.average=cumsum(sufficient.stat[4,])/(1:n.iter)
n3.average=cumsum(sufficient.stat[5,])/(1:n.iter)
n12.average=cumsum(sufficient.stat[6,])/(1:n.iter)

x11()
par(mfrow=c(3,2))
plot(n1.average,type="l",col="blue")
plot(n23.average,type="l",col="blue")
plot(n2.average,type="l",col="blue")
plot(n31.average,type="l",col="blue")
plot(n3.average,type="l",col="blue")
plot(n12.average,type="l",col="blue")

x11()
plot(xfin,cols=c("red","green","blue"))

#
# Fit a parametric model to the data with a profile analysis of the pseudo-likelihood ...
#
data(cells)
radius = data.frame(r=seq(0.05,0.15, by=0.01))
pradius = profilepl(radius, Strauss, cells)
plot(pradius)
mcells=ppm(cells,~1,Strauss(r=0.1),rbord=0.1)


#
# Residual analysis
#
data(japanesepines)
radius = data.frame(r=seq(0.05,0.11, by=0.01))
prStrauss = profilepl(radius, Strauss, japanesepines)
prAreaInter = profilepl(radius, AreaInter, japanesepines)

x11()
par(mfrow=c(1,2))
plot(prStrauss,main="Strauss - PL analysis")
plot(prAreaInter,main="AreaInter - PL analysis")

mj2=ppm(japanesepines,~1,Strauss(r=0.08),rbord=0.08)
mj3=ppm(japanesepines,~1,AreaInter(r=0.09),rbord=0.09)

r2=residuals(mj2,type="raw")
r3=residuals(mj3,type="raw")

x11()
par(mfrow=c(1,2))
plot(r2,main="")
plot(r3,main="")

q2=qqplot.ppm(mj2,type="raw")
q3=qqplot.ppm(mj3,type="raw")

x11()
par(mfrow=c(1,2))
plot(q2,main="")
plot(q3,main="")




